<?php

/**
 * GuestBook extension for Mediawiki.
 *
 * Copyright (C) 2005 - 2013 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * @package MediaWiki
 * @subpackage Extensions
 * @author Tino Reichardt <milky dash gbook at mcmilk dot de>
 */

if (!defined('MEDIAWIKI')) die();

/* here we define all routines for our guestbook */
class Guestbook extends SpecialPage {
    var $gbName, $gbOrigin, $gbEmail, $gbHomepage, $gbEntry, $gbCaptcha, $gbIP,
        $gbBrowser, $gbReplyto, $mySpecialPage, $gbNUM;

    function Guestbook() {
      SpecialPage::SpecialPage( "Guestbook" );
      $sp = Title::newFromText( "Special:Guestbook");
      $this->mySpecialPage = $sp->getLocalURL();
      $this->myAddPage = "Special:Guestbook";
      $this->includable(true);
      $this->listed(false);
    }

    function execute($par) {
      global $IP, $wgOut, $wgRequest, $wgUser, $myExtensionPath;
      global $GuestbookAlias, $GuestbookHeadline, $GuestbookMaxID;

      SpecialPage::SpecialPage( "Guestbook" );
      if (isset($GuestbookAlias[$par])) {
        /* aliases like: Special:Guestbook/FirstBook */
        $num = $GuestbookAlias[$par];
        $sp = Title::newFromText( "Special:Guestbook/$par");
        $this->mySpecialPage = $sp->getLocalURL();
        $this->myAddPage = "Special:Guestbook/$par";
        //echo "isALIAS($num)";
      } else if (ctype_digit($par)) {
        /* ID like: Special:Guestbook/1 */
        $num = $par;
        $sp = Title::newFromText( "Special:Guestbook/$num");
        $this->mySpecialPage = $sp->getLocalURL();
        $this->myAddPage = "Special:Guestbook/$num";
        //echo "isINT($num)";
      } else {
        /* all others are like Special:Guestbook/1 */
        $num = 1;
        $sp = Title::newFromText( "Special:Guestbook/$num");
        $this->mySpecialPage = $sp->getLocalURL();
        $this->myAddPage = "Special:Guestbook/$num";
        //echo "isDEF($num)";
      }

      if ($num > $GuestbookMaxID) {
        $num = 1;
        $sp = Title::newFromText( "Special:Guestbook/$num");
        $this->mySpecialPage = $sp->getLocalURL();
        $this->myAddPage = "Special:Guestbook/$num";
        //echo "isMAX($num)";
      }

      /* negativ values are new books (no replys to other entries) */
      $this->gbNUM = $num * -1;

      if (isset($GuestbookHeadline[$num])) {
        $wgOut->setPagetitle($GuestbookHeadline[$num]);
      } else {
        $wgOut->setPagetitle($this->text("headline"));
      }

      $wgOut->setArticleRelated(false); /* removes the getID "Fatal error" ;) */
      if (is_file("$IP/extensions/gbook/gbook${num}.css")) {
        $cssfile = "$myExtensionPath/gbook${num}.css";
      } else {
        $cssfile = "$myExtensionPath/gbook.css";
      }

      $wgOut->addLink(array(
        'rel'        => 'stylesheet',
        'type'       => 'text/css',
        'media'      => 'screen,projection',
        'href'       => $cssfile
      ));

      /* add some new entry */
      $this->checktable();

      if ($this->SafeString($wgRequest->getText( 'action' )) == "add") {
        if (!$this->ParseUserInput()) {
          $this->ShowSubmitForm($this->gbNUM);
          return;
        }
      }

      /* reply to some entry */
      if (is_numeric($this->SafeString($wgRequest->getText('reply')))) {
        if (!$this->ParseUserInput()) {
          $this->ShowSubmitForm($this->SafeString($wgRequest->getText( 'reply' )));
          return;
        }
      }

      /* moderate some entry */
      if ($wgUser->isAllowed('gbook')) {
        if ($this->SafeString($wgRequest->getText('mod')) != "") {
          if (!$this->ParseModInput()) {
            $this->ShowModForm();
            return;
          }
        }
      }

      /* fast deletion of spam entries */
      if ($wgUser->isAllowed('gbook')) {
        if ($this->SafeString($wgRequest->getText('del')) != "") {
          global $wgRequest;

          $id = $this->SafeString($wgRequest->getText('del'));
          if (!is_numeric($id)) {
            $this->error($this->text("err_noentry"));
          }

          /* get the old entry stuff from the database */
          $dbr =& wfGetDB( DB_SLAVE );
          $gb = $dbr->tableName('guestbook');
          $sql="DELETE FROM $gb WHERE gb_id=$id";
          $res = $dbr->query($sql, 'wfGuestbookSpecial');
          $this->success("SPAM entry with id=$id was deleted :-)");
          return;
        }
      }

      $p=$this->SafeString($wgRequest->getText( 'p' ));
      if (is_numeric($p)) {
        $this->ShowEntryPage($p);
      } else {
        $this->ShowEntryPage(0);
      }

      return;
    }

    function success($msg) {
      global $wgOut;

      $wgOut->addHTML('<div class="successbox">');
      $wgOut->addHTML($msg);
      $wgOut->addHTML('</div><div class="visualClear"></div>');

      return;
    }

    function error($msg) {
      global $wgOut;

      $wgOut->addHTML('<div class="errorbox">');
      $wgOut->addHTML($msg);
      $wgOut->addHTML('</div><div class="visualClear"></div>');

      return;
    }

    function checktable() {
      /* get the old entry stuff from the database */
      $dbw = wfGetDB( DB_MASTER );
      $gb = $dbw->tableName('guestbook');


      $sql="
      CREATE TABLE IF NOT EXISTS $gb (
        gb_id INT AUTO_INCREMENT,
        gb_replyto INT NOT NULL DEFAULT -1,
        gb_timestamp CHAR(14) BINARY,
        gb_name BLOB,
        gb_origin BLOB,
        gb_email BLOB,
        gb_homepage BLOB,
        gb_entry BLOB,
        gb_ip BLOB,
        gb_browser BLOB,
        PRIMARY KEY (gb_id)
      );";

      $dbw->query($sql, 'wfGuestbookSpecial');
      $res = $dbw->query('SHOW COLUMNS FROM ' . $gb, 'wfGuestbookSpecial');
      $obj = $dbw->fetchObject($res);
      $gb_replyto_exists = false; /* new since 0.8 */
      while ($obj = $dbw->fetchObject( $res )) {
        foreach($obj as $column) {
          if ($column == 'gb_replyto') {
            $gb_replyto_exists = true;
          }
        }
      }
      $dbw->freeResult($res);

      if (!$gb_replyto_exists) {
        $sql = "ALTER TABLE $gb ADD gb_replyto INT NOT NULL DEFAULT -1 AFTER gb_id";
        $res = $dbw->query($sql, 'wfGuestbookSpecial');
      }

      /**
       * since 0.14, we allow multiple guestbooks
       * - if gb_replyto = NULL; this was an entry in gbook... this changes now!
       * - if gb_replyto = -1 -> first gbook
       * - if gb_replyto = -2 -> second gbook
       * ...
       * -> we change all gb_replyto=NULL entries to -1 here...
       */
      $sql="SELECT COUNT(*) AS total FROM $gb WHERE gb_replyto is NULL";
      $res = $dbw->query($sql, 'wfGuestbookSpecial');
      $obj = $dbw->fetchObject( $res );
      $dbw->freeResult( $res );
      $total = $obj->total;
      if ($total > 0) {
        /* change entries of pre 0.14 database now ... */
        $sql = "UPDATE $gb SET gb_replyto = -1 WHERE gb_replyto is NULL";
        $dbw->query($sql, 'wfGuestbookSpecial');
      }

      return;
    }

    function text($str) {
      global $wgUser, $IP;

      /* we may add some other languages here as well */
      $lang = $wgUser->getOption('language');
      if (is_file("$IP/extensions/gbook/gbook.i18n-$lang.php")) {
        require("gbook.i18n-$lang.php");
      } else {
        require("gbook.i18n-en.php");
      }
      if (!isset($msg[$str])) {
        require("gbook.i18n-en.php");
        /* try english text for new stuff */
        if (!isset($msg[$str])) {
          return "error @ text($str)";
        }
      }

      return $msg[$str];
    }

    /* lock for bad characters ... */
    function SafeString($str) {
      global $BadRequestStrings;

      /* html tags */
      $str = preg_replace("/(<\/?)(\w+)([^>]*>)/", "", $str);
      return str_replace($BadRequestStrings, "", $str);
    }

    /* lock for spam */
    function TestForSpam($str) {
      global $wgSpamRegex;

      if (isset($wgSpamRegex))
        if (preg_match($wgSpamRegex, $str)) {
          $this->error($this->text("err_spam"));
          return 1; /* we have some spam... */
        }

      /* here, you can add more checks.... /TR 2012 */

      /* no spam */
      return 0;
    }

    /**
     * get user input (submitting new entries)
     */
    function ParseUserInput() {
      require_once("captcha-class.php");
      global $wgRequest, $want_captcha, $recaptcha_privkey, $wgUser;

      /* look, if the user is blocked */
      if ( $wgUser->isBlocked() ) {
        $this->error($this->text("err_blocked"));
        return;
      }

      /* we are called the first time */
      $this->gbReplyto = $this->SafeString($wgRequest->getText( 'gbReplyto' ));
      if ($this->TestForSpam($this->gbReplyto)) return;

      $this->gbSubmit = $this->SafeString($wgRequest->getText( 'gbSubmit' ));
      if ($this->gbSubmit == "") return;
      if ($this->TestForSpam($this->gbSubmit)) return;

      /* required */
      $this->gbName = $this->SafeString($wgRequest->getText( 'gbName' ));
      if ($this->TestForSpam($this->gbName)) return;

      $this->gbEntry = $this->SafeString($wgRequest->getText( 'gbEntry' ));
      if ($this->TestForSpam($this->gbEntry)) return;

      $this->gbCaptcha = $this->SafeString($wgRequest->getText( 'gbCaptcha' ));
      if ($this->TestForSpam($this->gbCaptcha)) return;

      /* optional */
      $this->gbOrigin = $this->SafeString($wgRequest->getText( 'gbOrigin' ));
      if ($this->TestForSpam($this->gbOrigin)) return;

      $this->gbEmail = $this->SafeString($wgRequest->getText( 'gbEmail' ));
      if ($this->TestForSpam($this->gbEmail)) return;

      $this->gbHomepage = $this->SafeString($wgRequest->getText( 'gbHomepage' ));
      if ($this->TestForSpam($this->gbHomepage)) return;

      if ($this->gbName == "") {
        $this->error($this->text("err_name"));
        return;
      }

      if ($this->gbEntry == "") {
        $this->error($this->text("err_entry"));
        return;
      }

      /* captcha on/off */
      if ($want_captcha) {

        if (isset($recaptcha_privkey)) {

          /* 1) try re-captcha if set */
          require_once('recaptchalib.php');
          $resp = gbook_recaptcha_check_answer($recaptcha_privkey,
            $_SERVER["REMOTE_ADDR"],
            $_POST["recaptcha_challenge_field"],
            $_POST["recaptcha_response_field"]);
          if (!$resp->is_valid) {
            /* What happens when the CAPTCHA was entered incorrectly */
            $this->error($this->text("captcha_fail"));
            return;
          }

        } else {

          /* 2) try own captcha if set */
          if ($this->gbCaptcha == "") {
            $this->error($this->text("err_captcha"));
            return;
          }

          $cap = new captcha();
          if ($cap->verify($this->gbCaptcha) != true) {
              $this->error($this->text("captcha_fail"));
              return;
          }
          unset($cap);
        }

      }

      $gbTimestamp = time(0);
      $dbw =& wfGetDB( DB_MASTER );
      $dbw->insert('guestbook', array(
        'gb_timestamp'  => $gbTimestamp,
        'gb_name' => $this->gbName,
        'gb_entry' => $this->gbEntry,
        'gb_email' => $this->gbEmail,
        'gb_origin' => $this->gbOrigin,
        'gb_homepage' => $this->gbHomepage,
        'gb_replyto' => $this->gbReplyto,
        'gb_ip' => $_SERVER['REMOTE_ADDR'],
        'gb_browser' => $_SERVER['HTTP_USER_AGENT']),
        'Guestbook::ParseUserInput', array('IGNORE'));

      $this->success($this->text("entry_okay"));
      return 1;
    }

    /**
     * get moderator input (editing/deleting old entries)
     */
    function ParseModInput() {
      require_once("captcha-class.php");
      global $wgRequest;

      $id = $this->SafeString($wgRequest->getText('mod'));
      if (!is_numeric($id)) {
        $this->error($this->text("err_noentry"));
      }

      /* get the old entry stuff from the database */
      $dbr =& wfGetDB( DB_SLAVE );
      $gb = $dbr->tableName('guestbook');

      $sql="SELECT * FROM $gb WHERE gb_id=$id";
      $res = $dbr->query($sql, 'wfGuestbookSpecial');
      $obj = $dbr->fetchObject($res);
      $dbr->freeResult($res);

      $this->gbName     = $obj->gb_name;
      $this->gbEntry    = $obj->gb_entry;
      $this->gbEmail    = $obj->gb_email;
      $this->gbOrigin   = $obj->gb_origin;
      $this->gbHomepage = $obj->gb_homepage;
      $this->gbIP       = $obj->gb_ip;
      $this->gbBrowser  = $obj->gb_browser;

      /* when called the first time, do nothing special */
      $this->gbSubmit = $this->SafeString($wgRequest->getText( 'gbSubmit' ));
      if ($this->gbSubmit == "") {
        return 0;
      }

      $todo = $this->SafeString($wgRequest->getText( 'todo' ));
      switch ($todo) {
      case $this->text("b_change"):
        $dbw =& wfGetDB( DB_MASTER );
        $dbw->update('guestbook',
         array(
          'gb_name'       => $this->SafeString($wgRequest->getText('gbName')),
          'gb_entry'      => $this->SafeString($wgRequest->getText('gbEntry')),
          'gb_email'      => $this->SafeString($wgRequest->getText('gbEmail')),
          'gb_origin'     => $this->SafeString($wgRequest->getText('gbOrigin')),
          'gb_homepage'   => $this->SafeString($wgRequest->getText('gbHomepage')),
          'gb_ip'         => $this->SafeString($wgRequest->getText('gbIP')),
          'gb_browser'    => $this->SafeString($wgRequest->getText('gbBrowser'))),
         array('gb_id' => "$id"),
          'Guestbook::ParseModInput', array('IGNORE'));
        $this->success($this->text("change_okay"));
        return 1;
      case $this->text("b_delete"):
        $sql="DELETE FROM $gb WHERE gb_id=$id";
        $res = $dbr->query($sql, 'wfGuestbookSpecial');
        $this->success($this->text("delete_okay"));
        return 1;
      }

      return 0;
    }

    function EntryExist($id) {
      global $wgOut, $wgUser;

      $dbr =& wfGetDB( DB_SLAVE );
      $gb = $dbr->tableName('guestbook');

      $sql = "SELECT gb_id FROM $gb WHERE gb_id=$id";
      $res = $dbr->query($sql, 'wfGuestbookSpecial');
      $obj = $dbr->fetchObject($res);
      $dbr->freeResult($res);

      return is_numeric($obj->gb_id);
    }

    function ShowEntry($id) {
      global $wgOut, $wgUser, $myExtensionPath, $GetRequestSeperator;

      $dbr =& wfGetDB( DB_SLAVE );
      $gb = $dbr->tableName('guestbook');

      $sql="SELECT * FROM $gb WHERE gb_id=$id";
      $res = $dbr->query($sql, 'wfGuestbookSpecial');
      while ($obj = $dbr->fetchObject( $res )) {
        $name=$obj->gb_name;
        $ts=date("Y-m-d @ H:i:s", $obj->gb_timestamp);
        $origin=$obj->gb_origin;
        $entry=str_replace("\n", "<br />", $obj->gb_entry);
        $browser=$obj->gb_browser;
        $ip=$obj->gb_ip;
        $id=$obj->gb_id;
        $replyto=$obj->gb_replyto;
        $b_reply=$this->text('b_reply');

        /* create to homepage link */
        $hp = $obj->gb_homepage;
        if ($hp != "") {
          if (strstr($hp,'://')) {
            $url = $hp;
          } else {
            $url = 'http://' . $hp;
          }
          $hp_link="(<a href=\"$url\">$hp</a>)";
        } else {
          $hp_link="";
        }

        /* create email link */
        $email=$obj->gb_email;
        if ($email != "") {
          $email=str_replace("@", "(at)", $email);
          $email=str_replace(".", "(dot)", $email);
          $email=str_replace("-", "(dash)", $email);
          $email_link="<a href=\"mailto:$email\"></a>";
        } else {
          $email_link="";
        }

        $moderate  = "";
        if ($wgUser->isAllowed('gbook')) {
          $moderate .= "<a href=\"" . "$this->mySpecialPage" . $GetRequestSeperator . "mod=$id\">";
          $moderate .= "<img alt=\"modify\" src=\"$myExtensionPath/gbook_tool.png\" />";
          $moderate .= "</a>";
          $moderate .= " <a target=\"_blank\" href=\"" . "$this->mySpecialPage" .  $GetRequestSeperator . "del=$id\">";
          $moderate .= "<img alt=\"modify\" src=\"$myExtensionPath/gbook_delete.png\" />";
          $moderate .= "</a> ";
        }
        $moderate .= "<a href=\"" . "$this->mySpecialPage" . $GetRequestSeperator . "reply=$id\">";
        $moderate .= "<img alt=\"$b_reply\" src=\"$myExtensionPath/gbook_reply.png\" />";
        $moderate .= "</a>";

        $out = <<<ENDFORM
<table width="100%" cellspacing="0" cellpadding="0">
<tr>
 <td width="100%"><b>$name</b> $email_link $hp_link</td>
 <td nowrap><small>$ts</small></td>
</tr><tr>
 <td colspan="2">
ENDFORM;
        $wgOut->addHTML($out);
        $wgOut->addWikiText($entry);
        $out = <<<ENDFORM
</td>
</tr>
</table>

<table width="100%" cellspacing="0" cellpadding="0"><tr>
 <td nowrap align="left">$moderate</td>
 <td align="right" width="100%" valign="bottom"><small>[$ip / $browser]</small></td>
</tr>
</table>
ENDFORM;
        $wgOut->addHTML($out);
        $this->ShowEntryReplies($id);
      }
      $dbr->freeResult( $res );
    }

    function ShowEntryReplies($id) {
      global $wgOut, $wgUser;

      $dbr =& wfGetDB( DB_SLAVE );
      $gb = $dbr->tableName('guestbook');

      $sql="SELECT gb_id FROM $gb WHERE gb_replyto=$id";
      $res = $dbr->query($sql, 'wfGuestbookSpecial');
      while ($obj = $dbr->fetchObject( $res )) {
        $wgOut->addHTML( '<div class="gbook">' );
        $this->ShowEntry($obj->gb_id);
        $wgOut->addHTML( '</div>' );
      }
    }

    /* page=0 means, begin with latest entries */
    function ShowEntryPage($page) {
      global $wgOut, $wgUser, $myExtensionPath, $GetRequestSeperator, $EntriesPerPage;

      $dbr =& wfGetDB( DB_SLAVE );
      $gb = $dbr->tableName('guestbook');

      $sql="SELECT COUNT(*) AS total FROM $gb WHERE gb_replyto=" . $this->gbNUM;
      $res = $dbr->query($sql, 'wfGuestbookSpecial');
      $obj = $dbr->fetchObject( $res );
      $dbr->freeResult( $res );
      $total = $obj->total;

      if (isset($EntriesPerPage)) {
        $limit = $EntriesPerPage;
      } else {
        $limit = $wgUser->getOption('rows');
      }

      $diff  = $total % $limit; /* entries in last page (oldest entries) */
      if ($diff != 0) {
        $pages = (int)($total / $limit);
        if ($page == 0) $page = $pages + 1;
        $start = $total - ($page * $limit) + ($limit - $diff);
      } else {
        $pages = (int)($total / $limit) - 1;
        if ($page == 0) $page = $pages + 1;
        $start = $total - ($page * $limit);
      }
      if ($start < 0) { $start=0; }

      $wgOut->addHTML( '<div class="gbook">' );
      $wgOut->addWikiText($this->text("new_entry"));

      if ($pages >= 1) {
        $wgOut->addWikiText("----");
        $wgOut->addHTML("<b>");
        for ($p=$pages+1; $p>0; $p--) {
          if ($p == $page) {
            $wgOut->addHTML("$p ");
          } else {
            $wgOut->addHTML("<a href=\"$this->mySpecialPage" . $GetRequestSeperator ."p=$p\">$p</a> ");
          }
        }
        $wgOut->addHTML("</b>");
      }
      $wgOut->addHTML( '</div>' );

      $sql="SELECT gb_id FROM $gb WHERE gb_replyto=" . $this->gbNUM . "
        ORDER BY gb_timestamp DESC LIMIT $start,$limit ";
      $res = $dbr->query($sql, 'wfGuestbookSpecial');
      while ($obj = $dbr->fetchObject( $res )) {
        $wgOut->addHTML( '<div class="gbook">' );
        $this->ShowEntry($obj->gb_id);
        $wgOut->addHTML( '</div>' );
      }

      if ($pages >= 1) {
        $wgOut->addHTML( '<div class="gbook">' );
        $wgOut->addHTML("<b>");
        for ($p=$pages+1; $p>0; $p--) {
          if ($p == $page) {
            $wgOut->addHTML("$p ");
          } else {
            $wgOut->addHTML("<a href=\"$this->mySpecialPage" . $GetRequestSeperator . "p=$p\">$p</a> ");
          }
        }
        $wgOut->addHTML("</b>");
        $wgOut->addHTML( '</div>' );
      }

      return;
    }

    /**
     * show the submitting form
     */
    function ShowSubmitForm($replyto) {
      global $wgOut, $wgUser, $myExtensionPath, $want_captcha, $recaptcha_pubkey;

      $gb1=$this->text("f_name");
      $gb2=$this->text("f_origin");
      $gb3=$this->text("f_email");
      $gb4=$this->text("f_homepage");
      $gb5=$this->text("f_entry");
      $gb6=$this->text("f_captcha");
      $gb7=$this->text("show_entries");
      $lnk="$this->mySpecialPage";
      $cap="$myExtensionPath/captcha.php";
      $alt=$this->text("captcha_desc");
      $b_reset=$this->text("b_reset");
      $b_submit=$this->text("b_submit");
      $hl=$this->text("hl_create");
      $lng=$wgUser->getOption('language');

      if ($replyto <= 0) {
        /* neuer eintrag */
        $replyto = $this->gbNUM;
      } else if ($replyto > 0) {
        if (!$this->EntryExist($replyto)) {
          $this->error($this->text("err_noentry"));
          unset($replyto);
        } else {
          $b_submit=$this->text("b_reply");
          $wgOut->addHTML( '<div class="gbook">' );
          $this->ShowEntry($replyto);
          $wgOut->addHTML( '</div>' );
        }
      }

      $out=<<<ENDFORM
<div id="gbook">
<h2>$hl</h2>
<form name="guestbook" method="post" action="">
<input type="hidden" name="gbSubmit" value="okay" />
<input type="hidden" name="gbReplyto" value="$replyto" />
<table width="100%" border="0">
<tr>
 <td valign="top">$gb1:</td>
 <td><input tabindex="1" type="text" name="gbName" size="40" value="$this->gbName" /></td>
ENDFORM;
 if ($want_captcha) {
   /* re-captcha */
   if (isset($recaptcha_pubkey)) {
   $out.=<<<ENDFORM
 <td valign="top" rowspan="4" align="right">
 <script type="text/javascript" src="http://www.google.com/recaptcha/api/challenge?k=$recaptcha_pubkey"> </script>
 <noscript>
  <iframe src="http://www.google.com/recaptcha/api/challenge?k=$recaptcha_pubkey" height="300" width="500" frameborder="0"></iframe><br/>
  <textarea name="recaptcha_challenge_field" rows="3" cols="40"></textarea>
  <input type="hidden" name="recaptcha_response_field" value="manual_challenge"/>
 </noscript>
ENDFORM;
   } else {
   /* own captcha */
   $out.=<<<ENDFORM
 <td valign="top" rowspan="4" align="right">
 <p><a href="http://$lng.wikipedia.org/wiki/Captcha" title="$alt">$gb6</a>: <input tabindex="6" type="text" name="gbCaptcha" size="8" value="" />
 <p><img alt="$alt" src="$cap"></img>
 </td>
ENDFORM;
   }
 } else {
   /* no captcha */
   $out.=<<<ENDFORM
   <td rowspan="4"></td>
ENDFORM;
 }
      $out.=<<<ENDFORM
</tr><tr>
 <td valign="top">$gb2:</td>
 <td><input tabindex="2" type="text" name="gbOrigin" size="40" value="$this->gbOrigin" /></td>
</tr><tr>
 <td valign="top">$gb3:</td>
 <td><input tabindex="3" type="text" name="gbEmail" size="40" value="$this->gbEmail" /></td>
</tr><tr>
 <td valign="top">$gb4:</td>
 <td><input tabindex="4" type="text" name="gbHomepage" size="40" value="$this->gbHomepage" /></td>
</tr><tr>
 <td valign="top">$gb5:</td>
 <td colspan="2"><textarea tabindex="5" name="gbEntry" cols="40" rows="7">$this->gbEntry</textarea></td>
</tr><tr>
 <td colspan="2"><a href="$lnk">$gb7</a></td>
 <td align="right">
  <input tabindex="7" type="submit" value="$b_submit"/>
  <input tabindex="8" type="reset" value="$b_reset"/>
 </td>
</tr>
</table>
</form>
</div>
ENDFORM;
      $wgOut->addHTML( $out );

      return;
    }

    /**
     * show the moderation form
     */
    function ShowModForm() {
      global $wgOut, $wgUser;

      $gb1=$this->text("f_name");
      $gb2=$this->text("f_origin");
      $gb3=$this->text("f_email");
      $gb4=$this->text("f_homepage");
      $gb5=$this->text("f_entry");
      $gb6=$this->text("f_ip");
      $gb7=$this->text("f_browser");
      $gb8=$this->text("show_entries");
      $lnk="$this->mySpecialPage";
      $b_change=$this->text("b_change");
      $b_delete=$this->text("b_delete");
      $b_reset=$this->text("b_reset");
      $hl=$this->text("hl_modify");
      $lng=$wgUser->getOption('language');

      $out=<<<ENDFORM
<div id="gbook">
<h2>$hl</h2>
<form name="guestbook" method="post" action="">
<input type="hidden" name="gbSubmit" value="okay" />
<input type="hidden" name="gbReplyto" value="$this->gbReplyto" />
<table width="100%" border="0">
<tr>
 <td valign="top">$gb1:</td>
 <td><input tabindex="1" type="text" name="gbName" size="40" value="$this->gbName" /></td>
</tr><tr>
 <td valign="top">$gb2:</td>
 <td><input tabindex="2" type="text" name="gbOrigin" size="40" value="$this->gbOrigin" /></td>
</tr><tr>
 <td valign="top">$gb3:</td>
 <td><input tabindex="3" type="text" name="gbEmail" size="40" value="$this->gbEmail" /></td>
</tr><tr>
 <td valign="top">$gb4:</td>
 <td><input tabindex="4" type="text" name="gbHomepage" size="40" value="$this->gbHomepage" /></td>
</tr><tr>
 <td valign="top">$gb5:</td>
 <td colspan="2"><textarea tabindex="5" name="gbEntry" cols="40" rows="7">$this->gbEntry</textarea></td>
</tr><tr>
 <td valign="top">$gb6:</td>
 <td><input tabindex="6" type="text" name="gbIP" size="40" value="$this->gbIP" /></td>
</tr><tr>
 <td valign="top">$gb7:</td>
 <td><input tabindex="7" type="text" name="gbBrowser" size="40" value="$this->gbBrowser" /></td>
</tr><tr>
 <td colspan="2"><a href="$lnk">$gb8</a></td>
 <td align="right" nowrap>
  <input name="todo" tabindex="8" type="submit" value="$b_change"/>
  <input tabindex="9" type="reset" value="$b_reset"/> |
  <input name="todo" tabindex="10" type="submit" value="$b_delete"/>
 </td>
</tr>
</table>
</form>
</div>
ENDFORM;
      $wgOut->addHTML( $out );

      return;
    }
} /* class */

?>
